# sage_setup: distribution = sagemath-environment
# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.5.45'
date = '2025-07-04'
banner = 'passagemath version 10.5.45, Release Date: 2025-07-04'
